@@ -8,10 +8,11 @@
 
 
 
+#include <config.h>
+
 #include <stdio.h>
 #include <string.h>
 
-#include "config.h"
 
 #include "slang.h"
 #include "_slang.h"
@@ -790,15 +791,16 @@ static int fast_extract_token(char **sp, char *t, int len)
    char *s = *sp;
    
    len = (unsigned char) *s++;
+   if (len <= 32) return 0;
    len -= 32;
-   if (len <= 0) return 0;
+   
    if (*s == '"')
      {
 	SLexpand_escaped_string (t, s, s + len);
      }
    else 
      { 
-	strncpy (t, s, len);
+	strncpy (t, s, (unsigned int) len);
 	t[len] = 0;
      }
    
@@ -1054,7 +1056,7 @@ static int get_token(void)
        && (Input != Terminate_String + 1))
      LLT->ofs = (int) (Input - LLT->buf);
    
-   while ((Input == NULL) || (0 == SLang_extract_token(&Input, Token, byte_comp)) 
+   while ((Input == NULL) || (0 == SLang_extract_token(&Input, Token, byte_comp))
 	  || (*Token == '%'))
      {
 	do
