@@ -7,29 +7,34 @@
  */
 
 
+#include <config.h>
+
 #include <stdio.h>
 #include <string.h>
 
-#include "config.h"
 
 #include "slang.h"
 
 #define SET_BIT(b, n) b[(unsigned int) (n) >> 3] |= 1 << ((unsigned int) (n) % 8)
 #define TEST_BIT(b, n) (b[(unsigned int)(n) >> 3] & (1 << ((unsigned int) (n) % 8)))
 #define LITERAL 1
-#define RANGE 2
-#define ANY 3
-#define BOL 4
-#define EOL 5
-#define NTH_MATCH 6
-#define OPAREN 7
-#define CPAREN 8
-#define ANY_DIGIT 9
-
-#define STAR 0x80
-#define LEAST_ONCE 0x40
-#define MAYBE_ONCE 0x20
-#define MANY 0x10
+#define RANGE 2			       /* [...] */
+#define ANY 3			       /* . */
+#define BOL 4			       /* ^ */
+#define EOL 5			       /* $ */
+#define NTH_MATCH 6		       /* \1 \2 ... \9 */
+#define OPAREN 7		       /* \( */
+#define CPAREN 0x8		       /* \) */
+#define ANY_DIGIT 0x9		       /* \d */
+#define BOW	0xA		       /* \< */
+#define EOW	0xB		       /* \> */
+#if 0
+#define NOT_LITERAL		0xC	       /* \~ */
+#endif
+#define STAR 0x80		       /* * */
+#define LEAST_ONCE 0x40		       /* + */
+#define MAYBE_ONCE 0x20		       /* ? */
+#define MANY 0x10		       /* {n,m} */
 /* The rest are additions */
 #define YES_CASE (STAR | BOL)
 #define NO_CASE  (STAR | EOL)
@@ -37,6 +42,9 @@
 #define UPPERCASE(x)  (cs ? (x) : UPPER_CASE(x))
 #define LOWERCASE(x)  (cs ? (x) : LOWER_CASE(x))
 
+static unsigned char Word_Chars[256];
+#define IS_WORD_CHAR(x) Word_Chars[(unsigned int) (x)]
+
 static int Next_Pos;
 static SLRegexp_Type *This_Reg;
 static unsigned char *This_Str;
@@ -52,7 +60,7 @@ static unsigned char *do_nth_match(int n, unsigned char *str, unsigned char *est
    if (n > (int) (estr - str)) return (NULL);
    
    /* This needs fixed for case sensitive match */
-   if (0 != strncmp((char *) str, (char *) bpos, n)) return (NULL);
+   if (0 != strncmp((char *) str, (char *) bpos, (unsigned int) n)) return (NULL);
    str += n;
    return (str);
 }
@@ -75,6 +83,18 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 
 	switch((unsigned char) p)
 	  {
+	   case BOW:
+	     if ((str != This_Str)
+		 && ((str >= estr)
+		     || IS_WORD_CHAR(*(str - 1))
+		     || (0 == IS_WORD_CHAR(*str)))) return NULL;
+	     break;
+	     
+	   case EOW:
+	     if ((str < estr)
+		 && IS_WORD_CHAR (*str)) return NULL;
+	     break;
+	     
 	   case YES_CASE: cs = 1; break;
 	   case NO_CASE: cs = 0; break;
 
@@ -83,10 +103,56 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 	     This_Reg->beg_matches[Next_Pos + 1] = -1;
 	     break;
 	   case CPAREN: 
-	     This_Reg->end_matches[Next_Pos] = (int) (str - This_Str) - This_Reg->beg_matches[Next_Pos];
+	     This_Reg->end_matches[Next_Pos] = (unsigned int) (str - This_Str) - This_Reg->beg_matches[Next_Pos];
 	     Next_Pos++;
 	     break;
+#ifdef NOT_LITERAL
+	   case NOT_LITERAL:
+	     if ((str >= estr) || (*buf == UPPERCASE(*str))) return (NULL);
+	     str++; buf++;
+	     break;
 	     
+	   case MAYBE_ONCE | NOT_LITERAL:
+	     save_str = str;
+	     if ((str < estr) && (*buf != UPPERCASE(*str))) str++;
+	     buf++;
+	     goto match_rest;
+
+	   case NOT_LITERAL | LEAST_ONCE:   /* match at least once */
+	     if ((str >= estr) || (UPPERCASE(*str) == UPPERCASE(*buf))) return (NULL);
+	     str++;
+	     /* drop */
+	   case STAR | NOT_LITERAL:
+	     save_str = str;  p1 = *buf;
+	     while ((str < estr) && (UPPERCASE(*str) != p1)) str++;
+	     buf++;
+	     goto match_rest;
+	     
+	     /* this type consists of the expression + two bytes that
+	        determine number of matches to perform */
+	   case MANY | NOT_LITERAL:
+	     p1 = *buf; buf++;
+	     n = n0 = (int) (unsigned char) *buf++;
+	     /* minimum number to match--- could be 0 */
+	     n1 = (int) (unsigned char) *buf++;
+	     /* maximum number to match */
+
+	     while (n && (str < estr) && (p1 != *str)) 
+	       {
+		  n--;
+		  str++;
+	       }
+	     if (n) return (NULL);
+	     
+	     save_str = str;
+	     n = n1 - n0;
+	     while (n && (str < estr) && (p1 != *str)) 
+	       {
+		  n--;
+		  str++;
+	       }
+	     goto match_rest;
+#endif				       /* NOT_LITERAL */
 	   case LITERAL: 
 	     if ((str >= estr) || (*buf != UPPERCASE(*str))) return (NULL);
 	     str++; buf++;
@@ -123,13 +189,15 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 		  str++;
 	       }
 	     if (n) return (NULL);
+	     
+	     save_str = str;
 	     n = n1 - n0;
 	     while (n && (str < estr) && (p1 == *str)) 
 	       {
 		  n--;
 		  str++;
 	       }
-	     break;
+	     goto match_rest;
 	     
 	   case NTH_MATCH: 
 	     if ((str = do_nth_match((int) (unsigned char) *buf, str, estr)) == NULL) return(NULL);
@@ -184,9 +252,30 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 	     buf += 32;
 	     goto match_rest;
 	     
+	     /* The first 32 bytes correspond to the range and the two 
+	      * following bytes indicate the min and max number of matches.
+	      */
 	   case MANY | RANGE:  
-	     /* needs finished */
-	     return (NULL);
+	     /* minimum number to match--- could be 0 */
+	     n = n0 = (int) (unsigned char) *(buf + 32);
+	     /* maximum number to match */
+	     n1 = (int) (unsigned char) *(buf + 33);
+
+	     while (n && (str < estr) && (TEST_BIT(buf, UPPERCASE(*str))))
+	       {
+		  n--;
+		  str++;
+	       }
+	     if (n) return (NULL);
+	     save_str = str;
+	     n = n1 - n0;
+	     while (n && (str < estr) && (TEST_BIT(buf, UPPERCASE(*str))))
+	       {
+		  n--;
+		  str++;
+	       }
+	     buf += 34;		       /* 32 + 2 */
+	     goto match_rest;
 	     
 	   case ANY_DIGIT:
 	     if ((str >= estr) || (*str > '9') || (*str < '0')) return (NULL);
@@ -267,11 +356,11 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 	return (NULL);
      }
    if ((p != 0) && (p != EOL)) return (NULL); else return (str);
-   
 }
 
 
-unsigned char *SLang_regexp_match(unsigned char *str, int len, SLRegexp_Type *reg)
+unsigned char *SLang_regexp_match(unsigned char *str, 
+				  unsigned int len, SLRegexp_Type *reg)
 {
    register unsigned char c = 0, *estr = str + len;
    int cs = reg->case_sensitive, lit = 0;
@@ -287,7 +376,7 @@ unsigned char *SLang_regexp_match(unsigned char *str, int len, SLRegexp_Type *re
      {
 	if (NULL == (epos = regexp_looking_at(str, estr, buf + 1, cs))) str = NULL;
 	reg->beg_matches[0] = (int) (str - This_Str);
-	reg->end_matches[0] = (int) (epos - str);
+	reg->end_matches[0] = (unsigned int) (epos - str);
 	return(str);
      }
 
@@ -324,7 +413,7 @@ unsigned char *SLang_regexp_match(unsigned char *str, int len, SLRegexp_Type *re
 	if (NULL != (epos = regexp_looking_at(str, estr, buf, cs)))
 	  {
 	     reg->beg_matches[0] = (int) (str - This_Str);
-	     reg->end_matches[0] = (int) (epos - str);
+	     reg->end_matches[0] = (unsigned int) (epos - str);
 	     return (str);
 	  }
 	str++;
@@ -364,18 +453,29 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
    int count, last_count, this_max_mm = 0, max_mm = 0, ordinary_search, 
      no_osearch = 0, min_length = 0;
    unsigned char *mm_p = NULL, *this_mm_p = NULL;
+   static int already_initialized;
    
    reg->beg_matches[0] = reg->end_matches[0] = 0;
    buf = reg->buf;
-   ebuf = reg->buf + reg->buf_len - 2; /* make some room */
+   ebuf = (reg->buf + reg->buf_len) - 2; /* make some room */
    pat = reg->pat;
    cs = reg->case_sensitive;
 
-   SLang_init_case_tables ();
+   if (already_initialized == 0)
+     {
+	SLang_init_case_tables ();
+#ifdef pc_system
+	SLmake_lut (Word_Chars, (unsigned char *) "_0-9a-zA-Z\200-\232\240-\245\341-\353", 0);
+#else
+	SLmake_lut (Word_Chars, (unsigned char *) "_0-9a-zA-Z\277-\326\330-\336\340-\366\370-\376", 0);
+#endif
+	already_initialized = 1;
+     }
    
    i = 1; while (i < 10) 
      {
-	reg->beg_matches[i] = reg->end_matches[i] = -1;
+	reg->beg_matches[i] = -1;
+	reg->end_matches[i] = 0;
 	i++;
      }
 
@@ -423,13 +523,32 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
 		  last = buf;
 		  *buf++ = NTH_MATCH; *buf++ = c;
 		  break;
-
+#ifdef NOT_LITERAL
+		case '~':	       /* slang extension */
+		  if ((c = *pat) == 0) ERROR;
+		  pat++;
+		  last = buf;
+		  *buf++ = NOT_LITERAL;
+		  *buf++ = c;
+		  min_length++;
+		  break;
+#endif
 		case 'd':	       /* slang extension */
 		  last = buf;
 		  *buf++ = ANY_DIGIT;
 		  min_length++;
 		  break;
 		  
+		case '<':
+		  last = NULL;
+		  *buf++ = BOW;
+		  break;
+		  
+		case '>':
+		  last = NULL;
+		  *buf++ = EOW;
+		  break;
+		  
 		case '{':
 		  if (last == NULL) goto literal_char;
 		  *last |= MANY;
@@ -437,6 +556,9 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
 		  if (tmppat == NULL) ERROR;
 		  pat = tmppat;
 		  *buf++ = n;
+
+		  min_length += (n - 1);
+		  
 		  if (*pat == '\\') 
 		    {
 		       *buf++ = n;
@@ -625,7 +747,7 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
    
    reg->osearch = ordinary_search;
    reg->must_match_str[15] = 0;
-   reg->min_length = (min_length > 0) ? min_length : 0;
+   reg->min_length = (min_length > 0) ? (unsigned int) min_length : 0;
    if (ordinary_search)
      {
 	strncpy((char *) reg->must_match_str, (char *) reg->pat, 15);
@@ -652,6 +774,44 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
    return(0);
 }
 
+char *SLregexp_quote_string (char *re, char *buf, unsigned int buflen)
+{
+   char ch;
+   char *b, *bmax;
+   
+   if (re == NULL) return NULL;
+   
+   b = buf;
+   bmax = buf + buflen;
+   
+   while (b < bmax)
+     {
+	switch (ch = *re++)
+	  {
+	   case 0:
+	     *b = 0;
+	     return buf;
+	     
+	   case '$':
+	   case '\\':
+	   case '[':
+	   case ']':
+	   case '.':
+	   case '^':
+	   case '*':
+	   case '+':
+	   case '?':
+	     *b++ = '\\';
+	    if (b == bmax) break;
+	     /* drop */
+	     
+	   default:
+	     *b++ = ch;
+	  }
+     }
+   return NULL;
+}
+
 /*
 #define MAX_EXP 4096
 int main(int argc, char **argv)
