@@ -7,9 +7,10 @@
  */
 
 
+#include <config.h>
+
 #include <stdio.h>
 
-#include "config.h"
 
 #include "slang.h"
 #include "_slang.h"
@@ -503,7 +504,13 @@ int SLang_read_line (SLang_RLine_Info_Type *rli)
      {
 	key = SLang_do_key (RL_Keymap, (int (*)(void)) rli->getkey);
 	
-	if ((key == NULL) || (key->f == NULL))
+	if ((key == NULL)
+#ifdef SLKEYMAP_OBSOLETE
+	    || (key->f == NULL)
+#else
+	    || (key->f.f == NULL)
+#endif
+	    )
 	  {
 	     rl_beep ();
 	  }
@@ -515,8 +522,13 @@ int SLang_read_line (SLang_RLine_Info_Type *rli)
 	       rl_eof_insert ();
 	     else if (key->type == SLKEY_F_INTRINSIC) 
 	       {
+#ifdef SLKEYMAP_OBSOLETE
 		  if (((int (*)(void))(key->f))())
 		    RLupdate (rli);
+#else
+		  if ((key->f.f)()) 
+		    RLupdate (rli);
+#endif
 	       }
 	
 	     if (SLang_Rline_Quit) 
@@ -530,7 +542,14 @@ int SLang_read_line (SLang_RLine_Info_Type *rli)
 		  return This_RLI->len;
 	       }
 	  }
-	if (key != NULL) This_RLI->last_fun = key->f;
+	if (key != NULL) 
+	  {
+#ifdef SLKEYMAP_OBSOLETE
+	     This_RLI->last_fun = key->f;
+#else
+	     This_RLI->last_fun = key->f.f;
+#endif
+	  }
      }
 }
 
@@ -561,8 +580,8 @@ static int rl_prev_line (void)
 {
    SLang_Read_Line_Type *prev;
    
-   if (((This_RLI->last_fun != (VOID *) rl_prev_line)
-	&& (This_RLI->last_fun != (VOID *) rl_next_line))
+   if (((This_RLI->last_fun != (FVOID_STAR) rl_prev_line)
+	&& (This_RLI->last_fun != (FVOID_STAR) rl_next_line))
        || (This_RLI->last == NULL))
      {
 	prev = This_RLI->tail;
@@ -588,8 +607,8 @@ static int rl_next_line (void)
 {
    SLang_Read_Line_Type *next;
    
-   if (((This_RLI->last_fun != (VOID *) rl_prev_line)
-	&& (This_RLI->last_fun != (VOID *) rl_next_line))
+   if (((This_RLI->last_fun != (FVOID_STAR) rl_prev_line)
+	&& (This_RLI->last_fun != (FVOID_STAR) rl_next_line))
        || (This_RLI->last == NULL))
       {
 	 rl_beep ();
@@ -641,61 +660,114 @@ int SLang_init_readline (SLang_RLine_Info_Type *rli)
 	
 	RL_Keymap->functions = SLReadLine_Functions;
 	
-   /* This breaks under some DEC ALPHA compilers (scary!) */
+	/* This breaks under some DEC ALPHA compilers (scary!) */
 #ifndef __DECC
-   for (ch = ' '; ch < 256; ch++)
-     {
-	simple[0] = (char) ch;
-	SLang_define_key1(simple, (VOID *) rl_self_insert, SLKEY_F_INTRINSIC, RL_Keymap);
-     }
+	for (ch = ' '; ch < 256; ch++)
+	  {
+	     simple[0] = (char) ch;
+# ifdef SLKEYMAP_OBSOLETE
+	     SLang_define_key1(simple, (VOID_STAR) rl_self_insert, SLKEY_F_INTRINSIC, RL_Keymap);
+# else
+	     SLkm_define_key (simple, (FVOID_STAR) rl_self_insert, RL_Keymap);
+# endif
+	  }
 #else
-   ch = ' ';
-   while (1)
-     {
-	simple[0] = (char) ch;
-	SLang_define_key1(simple, (VOID *) rl_self_insert, SLKEY_F_INTRINSIC, RL_Keymap);
-	ch = ch + 1;
-	if (ch == 256) break;
-     }
-#endif
+	ch = ' ';
+	while (1)
+	  {
+	     simple[0] = (char) ch;
+# ifdef SLKEYMAP_OBSOLETE
+	     SLang_define_key1(simple, (VOID_STAR) rl_self_insert, SLKEY_F_INTRINSIC, RL_Keymap);
+# else
+	     SLkm_define_key (simple, (FVOID_STAR) rl_self_insert, RL_Keymap);
+# endif
+	     ch = ch + 1;
+	     if (ch == 256) break;
+	  }
+#endif				       /* NOT __DECC */
 
 	simple[0] = SLang_Abort_Char;
-	SLang_define_key1(simple, (VOID *) rl_abort, SLKEY_F_INTRINSIC, RL_Keymap);
-	simple[0] = SLang_RL_EOF_Char;
-	SLang_define_key1(simple, (VOID *) rl_eof_insert, SLKEY_F_INTRINSIC, RL_Keymap);
-#ifndef pc_system	
-	SLang_define_key1 ("^[[A", (VOID *) rl_prev_line, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^[[B", (VOID *) rl_next_line, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^[[C", (VOID *) rl_right, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^[[D", (VOID *) rl_left, SLKEY_F_INTRINSIC, RL_Keymap);
+#ifdef SLKEYMAP_OBSOLETE
+	SLang_define_key1(simple, (VOID_STAR) rl_abort, SLKEY_F_INTRINSIC, RL_Keymap);
 #else
-	SLang_define_key1 ("^@H", (VOID *) rl_prev_line, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^@P", (VOID *) rl_next_line, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^@M", (VOID *) rl_right, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^@K", (VOID *) rl_left, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^@S", (VOID *) rl_del, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^@O", (VOID *) rl_eol, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^@G", (VOID *) rl_bol, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLkm_define_key (simple, (FVOID_STAR) rl_abort, RL_Keymap);
+#endif
+	simple[0] = SLang_RL_EOF_Char;
+#ifdef SLKEYMAP_OBSOLETE
+	SLang_define_key1(simple, (VOID_STAR) rl_eof_insert, SLKEY_F_INTRINSIC, RL_Keymap);
+#else
+	SLkm_define_key (simple, (FVOID_STAR) rl_eof_insert, RL_Keymap);
 #endif
-	SLang_define_key1 ("^E", (VOID *) rl_eol, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^I", (VOID *) rl_self_insert, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^A", (VOID *) rl_bol, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^M", (VOID *) rl_enter, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^K", (VOID *) rl_deleol, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^L", (VOID *) rl_deleol, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^V", (VOID *) rl_del, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^D", (VOID *) rl_del, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^F", (VOID *) rl_right, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^B", (VOID *) rl_left, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^?", (VOID *) rl_bdel, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^H", (VOID *) rl_bdel, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^P", (VOID *) rl_prev_line, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^N", (VOID *) rl_next_line, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^R", (VOID *) rl_redraw, SLKEY_F_INTRINSIC, RL_Keymap);
 	
-	SLang_define_key1 ("`", (VOID *) rl_quote_insert, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("\033\\", (VOID *) rl_trim, SLKEY_F_INTRINSIC, RL_Keymap);
+#ifndef pc_system
+# ifdef SLKEYMAP_OBSOLETE
+	SLang_define_key1 ("^[[A", (VOID_STAR) rl_prev_line, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^[[B", (VOID_STAR) rl_next_line, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^[[C", (VOID_STAR) rl_right, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^[[D", (VOID_STAR) rl_left, SLKEY_F_INTRINSIC, RL_Keymap);
+# else
+	SLkm_define_key  ("^[[A", (FVOID_STAR) rl_prev_line, RL_Keymap);
+	SLkm_define_key  ("^[[B", (FVOID_STAR) rl_next_line, RL_Keymap);
+	SLkm_define_key  ("^[[C", (FVOID_STAR) rl_right, RL_Keymap);
+	SLkm_define_key  ("^[[D", (FVOID_STAR) rl_left, RL_Keymap);
+# endif
+#else
+# ifdef SLKEYMAP_OBSOLETE
+	SLang_define_key1 ("^@H", (VOID_STAR) rl_prev_line, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^@P", (VOID_STAR) rl_next_line, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^@M", (VOID_STAR) rl_right, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^@K", (VOID_STAR) rl_left, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^@S", (VOID_STAR) rl_del, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^@O", (VOID_STAR) rl_eol, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^@G", (VOID_STAR) rl_bol, SLKEY_F_INTRINSIC, RL_Keymap);
+# else
+	SLkm_define_key  ("^@H", (FVOID_STAR) rl_prev_line, RL_Keymap);
+	SLkm_define_key  ("^@P", (FVOID_STAR) rl_next_line, RL_Keymap);
+	SLkm_define_key  ("^@M", (FVOID_STAR) rl_right, RL_Keymap);
+	SLkm_define_key  ("^@K", (FVOID_STAR) rl_left, RL_Keymap);
+	SLkm_define_key  ("^@S", (FVOID_STAR) rl_del, RL_Keymap);
+	SLkm_define_key  ("^@O", (FVOID_STAR) rl_eol, RL_Keymap);
+	SLkm_define_key  ("^@G", (FVOID_STAR) rl_bol, RL_Keymap);
+# endif
+#endif
+#ifdef SLKEYMAP_OBSOLETE
+	SLang_define_key1 ("^E", (VOID_STAR) rl_eol, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^I", (VOID_STAR) rl_self_insert, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^A", (VOID_STAR) rl_bol, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^M", (VOID_STAR) rl_enter, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^K", (VOID_STAR) rl_deleol, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^L", (VOID_STAR) rl_deleol, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^V", (VOID_STAR) rl_del, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^D", (VOID_STAR) rl_del, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^F", (VOID_STAR) rl_right, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^B", (VOID_STAR) rl_left, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^?", (VOID_STAR) rl_bdel, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^H", (VOID_STAR) rl_bdel, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^P", (VOID_STAR) rl_prev_line, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^N", (VOID_STAR) rl_next_line, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("^R", (VOID_STAR) rl_redraw, SLKEY_F_INTRINSIC, RL_Keymap);
 	
+	SLang_define_key1 ("`", (VOID_STAR) rl_quote_insert, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("\033\\", (VOID_STAR) rl_trim, SLKEY_F_INTRINSIC, RL_Keymap);
+#else
+	SLkm_define_key  ("^E", (FVOID_STAR) rl_eol, RL_Keymap);
+	SLkm_define_key  ("^I", (FVOID_STAR) rl_self_insert, RL_Keymap);
+	SLkm_define_key  ("^A", (FVOID_STAR) rl_bol, RL_Keymap);
+	SLkm_define_key  ("^M", (FVOID_STAR) rl_enter, RL_Keymap);
+	SLkm_define_key  ("^K", (FVOID_STAR) rl_deleol, RL_Keymap);
+	SLkm_define_key  ("^L", (FVOID_STAR) rl_deleol, RL_Keymap);
+	SLkm_define_key  ("^V", (FVOID_STAR) rl_del, RL_Keymap);
+	SLkm_define_key  ("^D", (FVOID_STAR) rl_del, RL_Keymap);
+	SLkm_define_key  ("^F", (FVOID_STAR) rl_right, RL_Keymap);
+	SLkm_define_key  ("^B", (FVOID_STAR) rl_left, RL_Keymap);
+	SLkm_define_key  ("^?", (FVOID_STAR) rl_bdel, RL_Keymap);
+	SLkm_define_key  ("^H", (FVOID_STAR) rl_bdel, RL_Keymap);
+	SLkm_define_key  ("^P", (FVOID_STAR) rl_prev_line, RL_Keymap);
+	SLkm_define_key  ("^N", (FVOID_STAR) rl_next_line, RL_Keymap);
+	SLkm_define_key  ("^R", (FVOID_STAR) rl_redraw, RL_Keymap);
+	SLkm_define_key  ("`", (FVOID_STAR) rl_quote_insert, RL_Keymap);
+	SLkm_define_key  ("\033\\", (FVOID_STAR) rl_trim, RL_Keymap);
+#endif	
 	if (SLang_Error) return -1;
      }
    
