@@ -6,10 +6,11 @@
  * License or the Perl Artistic License.
  */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <string.h>
 
-#include "config.h"
 #include "slang.h"
 #include "_slang.h"
 
@@ -50,6 +51,8 @@ static int This_Color;		       /* only the first 8 bits of this
 #define ALT_CHAR_FLAG 0x00
 #endif
 
+int SLsmg_Newline_Moves = 0;
+int SLsmg_Backspace_Moves = 0;
 
 static void blank_line (unsigned short *p, int n, unsigned char ch)
 {
@@ -88,6 +91,27 @@ void SLsmg_erase_eol (void)
    SL_Screen[This_Row].flags |= TOUCHED;
 }
 
+static void scroll_up (void)
+{
+   unsigned int i, imax;
+   unsigned short *neew;
+
+   neew = SL_Screen[0].neew;
+   imax = Screen_Rows - 1;
+   for (i = 0; i < imax; i++)
+     {
+	SL_Screen[i].neew = SL_Screen[i + 1].neew;
+	SL_Screen[i].flags |= TOUCHED;
+     }
+   SL_Screen[i].neew = neew;
+   SL_Screen[i].flags |= TOUCHED;
+   blank_line (neew, Screen_Cols, ' ');
+   This_Row--;
+}
+
+   
+
+
 void SLsmg_gotorc (int r, int c)
 {
    This_Row = r;
@@ -246,8 +270,16 @@ void SLsmg_write_nchars (char *str, int n)
    register unsigned short *p, old, neew;
    unsigned char ch;
    unsigned int flags;
-   int len, start_len = Start_Col, max_len;
-   char *str_max = str + n;
+   int len, start_len, max_len;
+   char *str_max;
+   int newline_flag;
+   
+   str_max = str + n;
+   
+   top:				       /* get here only on newline */
+   
+   newline_flag = 0;
+   start_len = Start_Col;
    
    if (point_visible (0) == 0) return;
    
@@ -305,7 +337,15 @@ void SLsmg_write_nchars (char *str, int n)
 		    }
 	       }
 	  }
-	else if (ch == '\n') break;
+	else if (ch == '\n')
+	  {
+	     newline_flag = 1;
+	     break;
+	  }
+	else if ((ch == 0x8) && SLsmg_Backspace_Moves)
+	  {
+	     if (len != 0) len--;
+	  }
 	else
 	  {
 	     if (ch & 0x80)
@@ -355,6 +395,28 @@ void SLsmg_write_nchars (char *str, int n)
    
    SL_Screen[This_Row].flags = flags;
    This_Col = len;
+   
+   if (SLsmg_Newline_Moves == 0)
+     return;
+   
+   if (newline_flag == 0)
+     {
+	while (str < str_max)
+	  {
+	     if (*str == '\n') break;
+	     str++;
+	  }
+	if (str == str_max) return;
+	str++;
+     }
+   
+   This_Row++;
+   This_Col = 0;
+   if (This_Row == Start_Row + Screen_Rows)
+     {
+	if (SLsmg_Newline_Moves > 0) scroll_up ();
+     }
+   goto top;
 }
 
 
@@ -399,7 +461,7 @@ static unsigned long compute_hash (unsigned short *s, int n)
 	sum += ch;
 	
 	h = sum + (h << 3);
-	if ((g = h & 0xE0000000L) != 0)
+	if ((g = h & 0xE0000000UL) != 0)
 	  {
 	     h = h ^ (g >> 24);
 	     h = h ^ g;
@@ -720,14 +782,19 @@ static void init_alt_char_set (void)
 }
 #endif
 
+static int Smg_Suspended;
 void SLsmg_suspend_smg (void)
 {
+   if (Smg_Suspended) return;
    SLtt_reset_video ();
+   Smg_Suspended = 1;
 }
 
 void SLsmg_resume_smg (void)
 {
    int i;
+   if (Smg_Suspended == 0) return;
+   Smg_Suspended = 0;
    SLtt_init_video ();
    Cls_Flag = 1;
    for (i = 0; i < Screen_Rows; i++)
@@ -969,3 +1036,45 @@ void SLsmg_fill_region (int r, int c, int dr, int dc, unsigned char ch)
    
    This_Row = r;
 }
+
+void SLsmg_forward (int n)
+{
+   This_Col += n;
+}
+
+void SLsmg_write_color_chars (unsigned short *s, unsigned int len)
+{
+   unsigned short *smax, sh;
+   char buf[32], *b, *bmax;
+   int color, save_color;
+   
+   smax = s + len;
+   b = buf;
+   bmax = b + sizeof (buf);
+   
+   save_color = This_Color;
+   while (s < smax)
+     {
+	sh = *s++;
+	
+	color = sh >> 8;
+	if ((color != This_Color) || (b == bmax))
+	  {
+	     if (b != buf) 
+	       {
+		  SLsmg_write_nchars (buf, (int) (b - buf));
+		  b = buf;
+	       }
+	     This_Color = color;
+	  }
+	*b++ = (char) (sh & 0xFF);
+     }
+   
+   if (b != buf)
+     SLsmg_write_nchars (buf, (int) (b - buf));
+	
+   This_Color = save_color;
+}
+
+	
+   
