@@ -8,10 +8,16 @@
 
 
 
+#include <config.h>
+
 #include <stdio.h>
+#include <time.h>
 
+#if defined(__GO32__) || defined(__WATCOMC__)
+# include <dos.h>
+# include <bios.h>
+#endif
 
-#include "config.h"
 #include "slang.h"
 #include "_slang.h"
 #include "slarray.h"
@@ -20,6 +26,7 @@
 #include <math.h>
 #endif
 
+#include <string.h>
 
 /* Standard intrinsic functions for S-Lang.  Included here are string
    and array operations */
@@ -101,7 +108,7 @@ static int pop_two_strings (char **a, char **b, int *free_a, int *free_b)
 static void SLdo_strcat(void)
 {
    char *a, *b, *c;
-   int len, lena;
+   unsigned int len, lena;
    int adata, bdata;
    
    if (pop_two_strings (&a, &b, &adata, &bdata)) return;
@@ -175,7 +182,7 @@ static int do_trim (char **beg, char **end, char *white)
 static void SLdo_strtrim(void)
 {
    char *a, *beg, *end, *c, *neew;
-   int len;
+   unsigned int len;
    int adata;
 
    if (SLang_pop_string(&a, &adata)) return;
@@ -203,7 +210,7 @@ static void SLdo_strcompress (void)
 {
    char *str, *white, *c;
    unsigned char *s, *beg, *end;
-   int len;
+   unsigned int len;
    int strdata, whitedata;
 
    if (pop_two_strings (&str, &white, &strdata, &whitedata)) return;
@@ -251,7 +258,7 @@ static int str_replace (void)
 {
    char *orig, *rep, *s, *newstr, *match;
    int free_orig, free_rep, free_match, ret;
-   int rep_len, match_len, new_len;
+   unsigned int rep_len, match_len, new_len;
    
    if (SLang_pop_string (&rep, &free_rep)) return 0;
    if (pop_two_strings (&orig, &match, &free_orig, &free_match))
@@ -343,7 +350,8 @@ static void SLquote_string (void)
 {
    char *str, *quotes, *q;
    int sdata, qdata;
-   int slash, n;
+   int slash;
+   unsigned int len;
    register char *t, *s, *q1;
    register unsigned char ch;
    
@@ -364,11 +372,11 @@ static void SLquote_string (void)
    
    /* calculate length */
    s = str;
-   n = 0;
-   while ((ch = (unsigned char) *s++) != 0) if (t[ch]) n++;
-   n += (int) (s - str);
+   len = 0;
+   while ((ch = (unsigned char) *s++) != 0) if (t[ch]) len++;
+   len += (unsigned int) (s - str);
    
-   if (NULL != (q = (char *) SLMALLOC(n)))
+   if (NULL != (q = (char *) SLMALLOC(len)))
      {
 	s = str; q1 = q;
 	while ((ch = (unsigned char) *s++) != 0)
@@ -420,9 +428,9 @@ static void SLdo_substr(void)
      {
 	n--;
 	if (m < 0) m = 0; else if (n + m > lena) m = lena - n;
-	b = SLmake_nstring (a + n, m);
+	b = SLmake_nstring (a + n, (unsigned int) m);
 	if (b != NULL)
-	  SLang_push_string(b);
+	  SLang_push_malloced_string (b);
      }
    if (adata == 1) SLFREE(a);
 }
@@ -431,7 +439,7 @@ static void SLdo_strsub(void)
 {
    char *a, *b;
    int adata, n, m;
-   int lena;
+   unsigned int lena;
    
    if (SLang_pop_integer(&m) || SLang_pop_integer(&n) || (SLang_pop_string(&a, &adata))) return;
    lena = strlen (a);
@@ -466,6 +474,22 @@ static void SLdo_strup(void)
      }
 }
 
+static int do_upper (void)
+{
+   int ch;
+   
+   if (SLang_pop_integer (&ch)) return -1;
+   return UPPER_CASE(ch);
+}
+
+static int do_lower (void)
+{
+   int ch;
+   
+   if (SLang_pop_integer (&ch)) return -1;
+   return LOWER_CASE(ch);
+}
+   
 static void SLdo_strlow(void)
 {
    unsigned char c, *a;
@@ -484,6 +508,100 @@ static void SLdo_strlow(void)
      }
 }
 
+static int do_strchop (void)
+{
+   int delim, quote, do_free, count;
+   char *str, *s0, *elm;
+   register char *s1;
+   register unsigned char ch;
+   int quoted;
+
+   if (SLang_pop_integer (&quote)
+       || (SLang_pop_integer (&delim))
+       || (SLang_pop_string (&str, &do_free)))
+     return 0;
+   
+   if ((quote < 0) || (quote > 255) 
+       || (delim <= 0) || (delim > 255))
+     {
+	SLang_Error = TYPE_MISMATCH;
+	if (do_free) SLFREE (str);
+	return 0;
+     }
+   
+   s1 = s0 = str;
+   
+   quoted = 0;
+   count = 0;
+   
+   while (1)
+     {
+	ch = (unsigned char) *s1;
+	if ((ch == quote) && quote)
+	  {
+	     s1++;
+	     quoted = 1;
+	     if (*s1 != 0) s1++;
+	  }
+	else if ((ch == delim) || (ch == 0))
+	  {
+	     if (NULL == (elm = SLmake_nstring (s0, (unsigned int)(s1 - s0))))
+	       break;
+
+	     /* No unquote it */
+	     if (quoted)
+	       {
+		  register char ch1, *p, *p1;
+		  
+		  p = p1 = elm;
+		  do
+		    {
+		       ch1 = *p1++;
+		       if (ch1 == '\\') ch1 = *p1++;
+		       *p++ = ch1;
+		    }
+		  while (ch1 != 0);
+		  quoted = 0;
+	       }
+	     
+	     SLang_push_malloced_string (elm);
+	     if (SLang_Error) break;
+	     count++;
+	     if (ch == 0) break;
+	     
+	     s1++;		       /* skip past delim */
+	     s0 = s1;		       /* and reset */
+	  }
+	else s1++;
+     }
+
+   if (do_free) SLFREE (str);
+   if (SLang_Error)
+     {
+	while (count != 0) 
+	  {
+	     count--;
+	     SLdo_pop ();
+	  }
+	count = 0;
+     }
+   return count;
+}
+
+static int do_strchopr (void)
+{
+   int count;
+   int zero = 0;
+   
+   count = do_strchop ();
+   if (count <= 0) return count;
+   
+   /* What was I thinking when I wrote SLreverse_stack !!! */
+   SLang_push_integer (count);
+   (void) SLreverse_stack (&zero);
+   return count;
+}
+
 static void SLdo_strcmp(void)
 {
    char *a, *b;
@@ -510,7 +628,7 @@ static void SLdo_strncmp(void)
 	SLang_Error = SL_INVALID_PARM;
      }
    else
-     SLang_push_integer (strncmp(a, b, n));
+     SLang_push_integer (strncmp(a, b, (unsigned int) n));
 
    if (adata) SLFREE(a);
    if (bdata) SLFREE(b);
@@ -643,7 +761,7 @@ char *SLstringize_object (SLang_Object_Type *obj)
       default:
 	cl = SLang_Registered_Types[stype];
 	if ((cl != NULL) && (cl->string != NULL)
-	    && (NULL != (s = (*cl->string) ((VOID *) obj->v.uobj))))
+	    && (NULL != (s = (*cl->string) ((VOID_STAR) obj->v.uobj))))
 	  {
 	     return s;
 	  }
@@ -685,7 +803,7 @@ static void lang_getenv_cmd(char *s)
    SLang_push_string(t);
 }
 
-#ifndef NO_PUTENV
+#ifdef HAVE_PUTENV
 static void lang_putenv_cmd(void)
 {
    char *s;
@@ -795,10 +913,11 @@ static char *SLdo_sprintf(char *fmt)
    unsigned char stmp, ttmp;
    long *varp;
    int var, want_width, width, precis, use_varp;
-   int len = 0, malloc_len = 0, dlen, do_free, guess_size;
+   unsigned int len = 0, malloc_len = 0, dlen;
+   int do_free, guess_size;
 #ifdef FLOAT_TYPE
    int tmp1, tmp2, use_float;
-   FLOAT x;
+   float64 x;
 #endif
    
    
@@ -808,7 +927,7 @@ static char *SLdo_sprintf(char *fmt)
 		  
 	/* p points at '%' */
 	
-	dlen = (int) (p - fmt);
+	dlen = (unsigned int) (p - fmt);
 	
 	if (len + dlen >= malloc_len)
 	  {
@@ -1002,7 +1121,7 @@ static char *SLdo_sprintf(char *fmt)
 
    if (out != NULL)
      {
-	outp = (char *) SLREALLOC(out, (int) (outp - out) + 1);
+	outp = (char *) SLREALLOC(out, (unsigned int) (outp - out) + 1);
 	if (outp != NULL) out = outp;
      }
    
@@ -1056,7 +1175,7 @@ static void SLsprintf(void)
 
 static void make_printable_string(char *s)
 {
-   int len;
+   unsigned int len;
    register char *s1 = s, ch, *ss1;
    char *ss;
    
@@ -1114,7 +1233,8 @@ char *SLang_extract_list_element(char *list, int *nth, int *delim)
 
 static int SLang_is_list_element(char *list, char *elem, int *delim)
 {
-   int d = *delim, n, len;
+   int d = *delim, n;
+   unsigned int len;
    char *l = list;
    
    len = strlen (elem);
@@ -1135,7 +1255,8 @@ static SLRegexp_Type regexp_reg;
 
 static int string_match (char *str, char *pat, int *np)
 {
-   int n = *np, len;
+   int n = *np;
+   unsigned int len;
    unsigned char rbuf[512], *match;
 
    regexp_reg.case_sensitive = 1;
@@ -1186,6 +1307,67 @@ static int my_system (char *s)
    return system (s);
 }
 
+#include <time.h>
+
+#if defined(__GO32__) || defined(__WATCOMC__)
+static char *djgpp_current_time (void)
+{
+   union REGS rg;
+   unsigned int year;
+   unsigned char month, day, weekday, hour, minute, sec;
+   char days[] = "SunMonTueWedThuFriSat";
+   char months[] = "JanFebMarAprMayJunJulAugSepOctNovDec";
+   static char the_date[26];
+   
+   rg.h.ah = 0x2A;
+#ifndef __WATCOMC__
+   int86(0x21, &rg, &rg);
+   year = rg.x.cx & 0xFFFF;
+#else
+   int386(0x21, &rg, &rg);
+   year = rg.x.ecx & 0xFFFF;
+#endif    
+  
+   month = 3 * (rg.h.dh - 1);
+   day = rg.h.dl;
+   weekday = 3 * rg.h.al;
+   
+   rg.h.ah = 0x2C;
+   
+#ifndef __WATCOMC__
+   int86(0x21, &rg, &rg);
+#else
+   int386(0x21, &rg, &rg);
+#endif
+  
+   hour = rg.h.ch;
+   minute = rg.h.cl;
+   sec = rg.h.dh;
+   
+   /* we want this form: Thu Apr 14 15:43:39 1994\n  */
+   sprintf(the_date, "%.3s %.3s%3d %02d:%02d:%02d %d\n",
+	   days + weekday, months + month, 
+	   day, hour, minute, sec, year);
+   return the_date;
+}
+#endif
+
+char *SLcurrent_time_string (void)
+{
+   char *the_time;
+#ifndef __GO32__
+   time_t myclock;
+   
+   myclock = time((time_t *) 0);
+   the_time = (char *) ctime(&myclock);
+#else
+   the_time = djgpp_current_time ();
+#endif
+   /* returns the form Sun Sep 16 01:03:52 1985\n\0 */
+   the_time[24] = '\0';
+   return(the_time);
+}
+
 /* These are here to let interpreter script have access to these values. */
 static int Integer_Type_Var = INT_TYPE;
 static int String_Type_Var = STRING_TYPE;
@@ -1240,6 +1422,25 @@ SLang_Name_Type SLang_Basic_Table[] =
     * 
     * Related Functions: @char@
     */
+   MAKE_INTRINSIC(".strchop", do_strchop, INT_TYPE, 0),
+   /* Prototype: Integer strchop (String str, Integer delim, Integer quote);
+    * This routine decomposes a string @str@ into a series of substrings 
+    * and returns the substrings to the stack along with the number of substrings.
+    * The string @str@ is assumed to consist of the substrings delimited 
+    * by @delim@.  The character @quote@ is assumed to quote the delimiter.
+    * For example,
+    * @ strchop ("apples,oranges,pears", ',', 0);
+    * will return @3@ to the top of the stack followed by the three strings:
+    * @"apples"@, @"oranges"@, and @"pears"@
+    * Related Functions: @strchopr@, @extract_element@
+    */
+   MAKE_INTRINSIC(".strchopr", do_strchopr, INT_TYPE, 0),
+   /* Prototype: Integer strchopr (String str, Integer delim, Integer quote);
+    * This routine performs exactly the same function as @strchop@ except 
+    * that it returns the substrings in the reverse order.
+    * Related Functions: @strchop@, @extract_element@
+    */
+     
    MAKE_INTRINSIC(".str_replace", str_replace, INT_TYPE, 0),
    /* Prototype: Integer str_replace (String a, String b, String c);
     * This function replaces the first occurance of @b@ in @a@ with @c@ and returns
@@ -1298,7 +1499,7 @@ SLang_Name_Type SLang_Basic_Table[] =
     * @     USE_ANSI_COLORS = 1;
     * @   }
     * Related Functions: @putenv@, @strlen@ */
-#ifndef NO_PUTENV
+#ifdef HAVE_PUTENV
    MAKE_INTRINSIC(".putenv",  lang_putenv_cmd, VOID_TYPE, 0),
    /* Prototype: Void putenv (String s);
     * This functions adds string @s@ to the environment.  Typically, @s@ should
@@ -1386,9 +1587,9 @@ SLang_Name_Type SLang_Basic_Table[] =
     * For example:
     * @  extract_element ("element 0, element 1, element 2", 1, ',');
     * returns the string @" element 1"@, whereas 
-    * @  extract_element ("element 0, element 1, element 2", 2, ' ');
+    * @  extract_element ("element 0, element 1, element 2", 1, ' ');
     * returns @"0,"@.
-    * Related Functions: @is_list_element@, @is_substr@.
+    * Related Functions: @is_list_element@, @is_substr@, @strchop@.
     */
    MAKE_INTRINSIC(".is_list_element", SLang_is_list_element, INT_TYPE, 3),
    /* Prototype: Integer is_list_element (String list, String elem, Integer delim);
@@ -1561,11 +1762,21 @@ SLang_Name_Type SLang_Basic_Table[] =
     * creates a 1 dimensional array of 10 floats and assigns it to @a@.
     */
    MAKE_INTRINSIC(".aget",  SLarray_getelem, VOID_TYPE, 0),
-   /* Syntax: i j ... k  ARRAY aget
-      returns ARRAY[i][j]...[k] */
+   /* Prototype: Object aget (Integer i, Integer j,..., Integer k, Array a);
+    * This function returns an element of the array @a@.  It is equivalent
+    * to the S-Lang expression @a[i, j,..., k]@.
+    * Note: This function should only be used if using S-Lang's RPN style 
+    * syntax.
+    * Related Functions: @aput@, @create_array@
+    */
    MAKE_INTRINSIC(".aput",  SLarray_putelem, VOID_TYPE, 0),
-   /* Syntax: x i j ... k  ARRAY put
-      sets ARRAY[i][j]...[k] = x */
+   /* Prototype: Void aput (Integer i, Integer j,..., Integer k, Array a);
+    * This function sets an element of the array @a@.  It is equivalent
+    * to the S-Lang expression @a[i, j,..., k]@.
+    * Note: This function should only be used if using S-Lang's RPN style 
+    * syntax.
+    * Related Functions: @aput@, @create_array@
+    */
 
    MAKE_INTRINSIC(".strncmp",  SLdo_strncmp, VOID_TYPE, 0),
    /* Prototype: Integer strncmp (String a, String b, Integer n);
@@ -1587,7 +1798,19 @@ SLang_Name_Type SLang_Basic_Table[] =
     * @ }
     * performs a case insensitive comparison operation on two strings by
     * converting them to lower case first.
-    * Related Functions: @strup@, @strcmp@, @strtrim@, @define_case@
+    * Related Functions: @strup@, @tolower@, @strcmp@, @strtrim@, @define_case@
+    */
+   MAKE_INTRINSIC(".tolower", do_lower, INT_TYPE, 0),
+   /* Prototype: Integer lower (Integer ch);
+    * This function takes an integer @ch@ and returns its lowercase
+    * equivalent.
+    * Related Functions: @toupper@, @strup@, @strlow@, @int@, @char@, @define_case@
+    */
+   MAKE_INTRINSIC(".toupper", do_upper, INT_TYPE, 0),
+   /* Prototype: Integer upper (Integer ch);
+    * This function takes an integer @ch@ and returns its uppercase
+    * equivalent.
+    * Related Functions: @tolower@, @strup@, @strlow@, @int@, @char@, @define_case@
     */
    MAKE_INTRINSIC(".strup", SLdo_strup, VOID_TYPE, 0),
    /* Prototype: String strup (String s);
@@ -1600,7 +1823,7 @@ SLang_Name_Type SLang_Basic_Table[] =
     * @ }
     * performs a case insensitive comparison operation on two strings by
     * converting them to upper case first.
-    * Related Functions: @strlow@, @strcmp@, @strtrim@, @define_case@
+    * Related Functions: @strlow@, @toupper@, @strcmp@, @strtrim@, @define_case@
     */
    MAKE_INTRINSIC(".isdigit",  SLdo_isdigit, INT_TYPE, 1),
    /* Prototype: Integer isdigit (String s);
@@ -1688,6 +1911,25 @@ SLang_Name_Type SLang_Basic_Table[] =
     * alter the stack and it is usually used for debugging purposes.
     * Related Functions: @_stkdepth@, @string@
     */
+   MAKE_INTRINSIC("._stk_roll", SLroll_stack, VOID_TYPE, 1),
+   /* Prototype: Void _stk_roll (Integer n);
+    * If @n@ is positive, the top @n@ items on the stack are rotated up.  If
+    * @n@ is negative, the top @abs(n)@ items on the stack are rotated down.
+    * For example, if the stack looks like:
+    * @ item-0
+    * @ item-1
+    * @ item-2
+    * @ item-3
+    * where @item-0@ is at the top of the stack, then @_stk_roll(-3)@ will
+    * change the stack to:
+    * @ item-2
+    * @ item-0
+    * @ item-1
+    * @ item-3
+    * Note that this function only has an effect for @abs(n) > 1@.
+    * Related Functions: @_stkdepth@, @print_stack@
+    */
+
    MAKE_INTRINSIC(".Sprintf", SLsprintf, VOID_TYPE, 0),
    /* Prototype:  String Sprintf(String format, ..., Integer n);
     * Sprintf formats a string from @n@ objects according to @format@.  
@@ -1799,7 +2041,7 @@ SLang_Name_Type SLang_Basic_Table[] =
     * @   variable b = create_array (array_info (a));
     * @   return copy_array (b, a);
     * @ }
-    * See also: @create_array@, @copy_array@
+    * Related Functions: @create_array@, @copy_array@
     */
    MAKE_INTRINSIC("._slang_guess_type", SLang_guess_type, INT_TYPE, 1),
    /* Prototype: Integer _slang_guess_type (String s);
@@ -1815,6 +2057,11 @@ SLang_Name_Type SLang_Basic_Table[] =
     * Related Functions: @integer@, @string@, @float@
     * Related Variables: @INT_TYPE@, @STRING_TYPE@, @FLOAT_TYPE@
     */
+   MAKE_INTRINSIC(".time", SLcurrent_time_string, STRING_TYPE, 0),
+   /* Prototype: String time ();
+    * This function returns the current time as a string in the form:
+    * @ Sun Apr 21 13:34:17 1996
+    */
    MAKE_VARIABLE("._slang_version", &SLang_Version, INT_TYPE, 1),
    MAKE_VARIABLE(".INT_TYPE", &Integer_Type_Var, INT_TYPE, 1),
    MAKE_VARIABLE(".STRING_TYPE", &String_Type_Var, INT_TYPE, 1),
@@ -1824,166 +2071,6 @@ SLang_Name_Type SLang_Basic_Table[] =
    SLANG_END_TABLE
 };
 
-#ifdef FLOAT_TYPE
-/* doing it this way means that I can handle things like 2.0 + 3 and
- * 3 + 2.0 at the same time.
- */
-static int float_bin_op (int op, unsigned char sta, unsigned char stb, 
-			 FLOAT *ap, FLOAT *bp)
-{
-   FLOAT c, a, b;
-   int ic;
-   
-   if (sta == FLOAT_TYPE) a = *ap;
-   else a = (FLOAT) *(int *) ap;
-   
-   if (stb == FLOAT_TYPE) b = *bp;
-   else b = (FLOAT) *(int *) bp;
-
-   switch (op)
-     {
-      case SLANG_PLUS:
-	c = a + b; goto push_float;
-      case SLANG_MINUS: c = a - b; goto push_float;
-      case SLANG_TIMES: c = a * b; goto push_float;
-      case SLANG_DIVIDE:
-	if (b == 0.0) 
-	  {
-	     SLang_Error = DIVIDE_ERROR;
-	     return 1;
-	  }
-	c = a / b; goto push_float;
-	
-      case SLANG_EQ: ic = (a == b); goto push_int;
-      case SLANG_NE: ic = (a != b); goto push_int;
-      case SLANG_GT: ic = (a > b); goto push_int;
-      case SLANG_GE: ic = (a >= b); goto push_int;
-      case SLANG_LT: ic = (a < b); goto push_int;
-      case SLANG_LE: ic = (a <= b); goto push_int;
-      default:
-	return 1;
-     }
-   
-   push_float:
-   SLang_push_float (c);
-   return 1;
-   
-   push_int:
-   SLang_push_integer (ic);
-   return 1;
-}	
-
-#endif
-
-static int int_bin_op (int op, unsigned char sta, unsigned char stb, 
-		       int *ap, int *bp)
-{
-   int c, a = *ap, b = *bp;
-   (void) sta;
-   (void) stb;
-   switch (op)
-     {
-      case SLANG_PLUS:
-	c = a + b; break;
-      case SLANG_MINUS: c = a - b; break;
-      case SLANG_TIMES: c = a * b; break;
-      case SLANG_DIVIDE:
-	if (b == 0) 
-	  {
-	     SLang_Error = DIVIDE_ERROR;
-	     return 1;
-	  }
-	c = a / b; break;
-
-      case SLANG_EQ: c = (a == b); break;
-      case SLANG_NE: c = (a != b); break;
-      case SLANG_GT: c = (a > b); break;
-      case SLANG_GE: c = (a >= b); break;
-      case SLANG_LT: c = (a < b); break;
-      case SLANG_LE: c = (a <= b); 
-	break;
-      default:
-	return 0;
-     }
-   SLang_push_integer (c);
-   return 1;
-}	
-
-static int unary_op_function (int op, unsigned char type, VOID *xp)
-{
-   int x;
-#ifdef FLOAT_TYPE 
-   FLOAT xf;
-   
-   if (type == FLOAT_TYPE) 
-     {
-	xf = *(FLOAT *) xp;
-	switch (op)
-	  {
-	   case SLANG_CHS: xf = -xf; break;
-	   case SLANG_SQR: xf = xf * xf; break;
-	   case SLANG_MUL2: xf = xf * 2.0; break;
-	   case SLANG_ABS: xf = (FLOAT) fabs((double) xf); break;
-	   case SLANG_SIGN: 
-	     if (xf > 0.0) x = 1;
-	     else if (xf < 0.0) x = -1;
-	     else x = 0;
-	     SLang_push_integer (x);
-	     return 1;
-	     
-	   default:
-	     return 0;
-	  }
-	SLang_push_float (xf);
-     }
-   else
-     {
-#else
-	(void) type;
-#endif
-	x = *(int *) xp;
-	switch (op)
-	  {
-	   case SLANG_CHS: x = -x; break;
-	   case SLANG_SQR: x = x * x; break;
-	   case SLANG_MUL2: x = x * 2; break;
-	   case SLANG_ABS: x = abs(x); break;
-	   case SLANG_SIGN: 
-	     if (x > 0) x = 1; else if (x < 0) x = -1;
-	   default:
-	     return 0;
-	  }
-	SLang_push_integer (x);
-#ifdef FLOAT_TYPE
-     }
-#endif
-   return 1;
-}
-
-static int register_types (void)
-{
-   if ((!SLang_register_class (ARRAY_TYPE, (VOID *) SLarray_free_array, NULL))
-       || !SLang_register_class (INT_TYPE, NULL, NULL)
-#ifdef FLOAT_TYPE
-       || !SLang_register_class (FLOAT_TYPE, NULL, NULL)
-#endif
-       )
-     return 0;
-   
-
-   if (!SLang_add_binary_op (INT_TYPE, INT_TYPE, (VOID *) int_bin_op)
-       || !SLang_add_unary_op (INT_TYPE, (VOID *) unary_op_function)
-#ifdef FLOAT_TYPE
-       || !SLang_add_binary_op (FLOAT_TYPE, FLOAT_TYPE, (VOID *) float_bin_op)
-       || !SLang_add_binary_op (FLOAT_TYPE, INT_TYPE, (VOID *) float_bin_op)
-       || !SLang_add_unary_op (FLOAT_TYPE, (VOID *) unary_op_function)
-#endif
-       )
-     return 0;
-   
-   return 1;
-}
-
 
 int init_SLang()
 {
@@ -2008,7 +2095,7 @@ int init_SLang()
    if (SLang_Name_Table == NULL) return 0;
 #endif
    
-   if (0 == register_types ()) return 0;
+   if (-1 == SLregister_types ()) return 0;
 
    if (!SLang_add_table(SLang_Basic_Table, "_Basic")) return(0);
    SLadd_variable(SLANG_SYSTEM_NAME);
