@@ -22,11 +22,12 @@
  * License or the Perl Artistic License.
 \*----------------------------------------------------------------------*/
 
+#include <config.h>
+
 #include <stdio.h>
 
-#include "config.h"
 
-#ifndef NO_STDLIB_H
+#ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
 #include <string.h>
@@ -148,18 +149,20 @@ int SLang_extract_token (char **linep, char *word_parm, int byte_comp)
    
    word_max = word + 250;
    
-    line = *linep;
+   line = *linep;
 
-    /* skip white space */
-    while (ch = *line++, (ch == ' ') || (ch == '\t'));
-
-    if ((!ch) || (ch == '\n'))
-      {
-	 *linep = line;
-	 return(0);
-      }
+   /* skip white space */
+   while (((ch = *line) == ' ') 
+	  || (ch == '\t')) line++;
 
+   if ((!ch) || (ch == '\n'))
+     {
+	*linep = line;
+	return(0);
+     }
+   
    *word++ = ch;
+   line++;
    
    /* Look for -something and rule out --something and -= something */
    if ((ch == '-') && 
